package org.eclipse.tptp.chat.dao;

import java.io.IOException;
import java.io.InputStream;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Vector;

import org.apache.commons.digester.Digester;
import org.eclipse.tptp.chat.config.ChatConfig;
import org.eclipse.tptp.chat.data.Message;
import org.eclipse.tptp.chat.data.Room;
import org.eclipse.tptp.chat.data.User;
import org.xml.sax.SAXException;

public class ChatDAO {

	private static ChatDAO instance;

	private LinkedHashMap rooms = new LinkedHashMap();

	private Vector users = new Vector();

	private ChatDAO() {
	}

	public static ChatDAO getInstance() {
		if (instance == null) {
			instance = new ChatDAO();
			instance.init(null);
		}
		return instance;

	}

	public synchronized void init(InputStream isConfigFile) {
		if (isConfigFile != null) {
			Digester digester = new Digester();
			digester.setValidating(false);
			digester.push(this);
			digester.addObjectCreate("rooms/room",
					"org.eclipse.tptp.chat.data.Room");
			digester.addSetProperties("rooms/room");
			digester.addSetNext("rooms/room", "addRoom");
			try {
				digester.parse(isConfigFile);
			} catch (IOException ioe) {
				ioe.printStackTrace();
			} catch (SAXException se) {
				se.printStackTrace();
			}
		}

	}

	public synchronized void addRoom(Room inRoom) {
		rooms.put(inRoom.getName(), inRoom);
	}

	public synchronized boolean isUsernameInUse(String inUsername) {

		boolean retVal = false;
		for (Iterator it = users.iterator(); it.hasNext();) {
			User user = (User) it.next();
			if (inUsername.equalsIgnoreCase(user.getUsername())) {
				retVal = true;
			}
		}
		return retVal;

	}

	public synchronized void logUserOut(User inUser) {
		String usernameToLogOut = inUser.getUsername();
		int i = 0;
		int indexToRemove = -1;
		for (Iterator it = users.iterator(); it.hasNext();) {
			User user = (User) it.next();
			if (usernameToLogOut.equalsIgnoreCase(user.getUsername())) {
				indexToRemove = i;
				break;
			}
			i++;
		}
		if (indexToRemove != -1) {
			users.remove(indexToRemove);
		}
	}

	public synchronized void logUserIn(User inUser) {
		users.add(inUser);
	}

	public synchronized Vector getRoomList() {

		Vector roomList = new Vector();
		for (Iterator it = rooms.values().iterator(); it.hasNext();) {
			roomList.add(it.next());
		}
		return roomList;

	}

	public synchronized void removeInactiveUsers() {

		Vector usersToRemove = new Vector();
		for (Iterator it = users.iterator(); it.hasNext();) {
			User user = (User) it.next();
			long now = new Date().getTime();
			long lastAJAXRequest = user.getLastRequest().getTime();
			if ((now - lastAJAXRequest) >= (ChatConfig
					.getUserInactivitySeconds() * 1000)) {
				usersToRemove.add(user);
			}
		}
		for (Iterator it = usersToRemove.iterator(); it.hasNext();) {
			User user = (User) it.next();
			removeUserFromAllRooms(user);
			logUserOut(user);
		}

	}

	public synchronized void removeUserFromAllRooms(User inUser) {

		for (Iterator it = rooms.keySet().iterator(); it.hasNext();) {
			String roomName = (String) it.next();
			Room room = (Room) rooms.get(roomName);
			room.removeUser(inUser);
		}

	}

	public synchronized void addUserToRoom(String inRoom, User inUser) {
		Room room = (Room) rooms.get(inRoom);
		room.addUser(inUser);
	}

	public synchronized Vector getUserList(String inRoom) {
		Vector userList = null;
		Room room = (Room) rooms.get(inRoom);
		userList = room.getUserList();
		return userList;

	}

	public synchronized Vector getMessages(String inRoom, Date inDateTime) {

		Room room = (Room) rooms.get(inRoom);
		return room.getMessages(inDateTime);

	}

	public synchronized void postMessage(String inRoom, Message inMessage) {

		Room room = (Room) rooms.get(inRoom);
		room.postMessage(inMessage);

	}

	public synchronized void removeUserFromRoom(String inRoom, User inUser) {

		Room room = (Room) rooms.get(inRoom);
		room.removeUser(inUser);

	}

	public synchronized LinkedHashMap getRoomUserCounts() {
		LinkedHashMap roomList = new LinkedHashMap();
		for (Iterator it = rooms.keySet().iterator(); it.hasNext();) {
			String roomName = (String) it.next();
			roomList.put(roomName, new Integer(((Room) rooms.get(roomName))
					.getUserList().size()));
		}
		return roomList;

	}
}
